@@ -23,8 +23,10 @@
 
 #include <libnfnetlink/libnfnetlink.h>
 #include <libnetfilter_conntrack/libnetfilter_conntrack.h>
+#include <libnetfilter_conntrack/libnetfilter_conntrack_tcp.h>
+#include <libnetfilter_conntrack/libnetfilter_conntrack_sctp.h>
+#include <libnetfilter_conntrack/libnetfilter_conntrack_dccp.h>
 
-#include "internal/deprecated.h"
 #include "internal/object.h"
 #include "internal/prototypes.h"
 #include "internal/types.h"
@@ -75,4 +77,8 @@
 #define likely(x)       __builtin_expect((x),1)
 #define unlikely(x)     __builtin_expect((x),0)
 
+#ifndef NSEC_PER_SEC
+#define NSEC_PER_SEC    1000000000L
+#endif
+
 #endif
