@@ -15,21 +15,32 @@ struct nfct_handle {
 	struct nfnl_subsys_handle	*nfnlssh_ct;
 	struct nfnl_subsys_handle	*nfnlssh_exp;
 
-	/* deprecated old API */
-	nfct_callback 			callback;
-	void 				*callback_data;
-	nfct_handler			handler;
-
 	/* callback handler for the new API */
-	struct nfnl_callback		nfnl_cb;
+	struct nfnl_callback		nfnl_cb_ct;
+	struct nfnl_callback		nfnl_cb_exp;
 
 	int			(*cb)(enum nf_conntrack_msg_type type, 
 				      struct nf_conntrack *ct,
 				      void *data);
 
+	/* This is the second version of the callback that includes
+	 * the Netlink header. This is the result of an early design
+	 * error, hiding Netlink details is evil. You end needing some
+	 * internal information at some point like the Netlink PortID. */
+	int			(*cb2)(const struct nlmsghdr *nlh,
+				       enum nf_conntrack_msg_type type,
+				       struct nf_conntrack *ct,
+				       void *data);
+
 	int			(*expect_cb)(enum nf_conntrack_msg_type type, 
 					     struct nf_expect *exp,
 					     void *data);
+
+	/* second version of the expect callback: it includes netlink header */
+	int			(*expect_cb2)(const struct nlmsghdr *nlh,
+					      enum nf_conntrack_msg_type type, 
+					      struct nf_expect *exp,
+					      void *data);
 };
 
 /* container used to pass data to nfnl callbacks */
@@ -58,6 +69,9 @@ union __nfct_l4_src {
 	struct {
 		u_int16_t 	port;
 	} sctp;
+	struct {
+		u_int16_t 	port;
+	} dccp;
 };
 
 union __nfct_l4_dst {
@@ -75,6 +89,9 @@ union __nfct_l4_dst {
 	struct {
 		u_int16_t 	port;
 	} sctp;
+	struct {
+		u_int16_t 	port;
+	} dccp;
 };
 
 union __nfct_address {
@@ -110,12 +127,17 @@ union __nfct_protoinfo {
 			u_int8_t 	value;
 			u_int8_t 	mask;
 		} flags[__DIR_MAX];
+		u_int8_t		wscale[__DIR_MAX];
 	} tcp;
 	struct {
 		u_int8_t 		state;
 		u_int32_t 		vtag[__DIR_MAX];
 	} sctp;
-
+	struct {
+		u_int8_t 		state;
+		u_int8_t		role;
+		u_int64_t		handshake_seq;
+	} dccp;
 };
 
 struct __nfct_counters {
@@ -137,16 +159,29 @@ struct nf_conntrack {
 	u_int32_t 	status;
 	u_int32_t	use;
 	u_int32_t	id;
+	u_int16_t	zone;
 
-#define __NFCT_HELPER_NAMELEN		30	/* same length in xt_helper */
+/* xt_helper uses a length size of 30 bytes, however, no helper name in
+ * the tree has exceeded 16 bytes length. Since 2.6.29, the maximum
+ * length accepted is 16 bytes, this limit is enforced during module load. */
+#define __NFCT_HELPER_NAMELEN		16
 	char 		helper_name[__NFCT_HELPER_NAMELEN];
+/* According to Eric Paris <eparis@redhat.com> this field can be up to 4096
+ * bytes long. For that reason, we allocate this dynamically. */
+	char		*secctx;
 
 	union __nfct_protoinfo 	protoinfo;
 	struct __nfct_counters 	counters[__DIR_MAX];
 	struct __nfct_nat 	snat;
 	struct __nfct_nat 	dnat;
 
-#define __NFCT_BITSET			2
+	struct {
+		u_int64_t	start;
+		u_int64_t	stop;
+	} timestamp;
+
+/* we've got more than 64 attributes now, we need 96 bits to store them. */
+#define __NFCT_BITSET			3
 	u_int32_t               set[__NFCT_BITSET];
 };
 
@@ -206,6 +241,19 @@ struct nfct_filter {
 		u_int32_t 	mask;
 	} l3proto[2][__FILTER_ADDR_MAX];
 
+	/*
+	 * FIXME: For IPv6 filtering, up to 20 IPs/masks (12 BSF lines
+	 * per comparison). I think that it is not worthy to try to support
+	 * more than that for performance reasons. It seems that oprofile
+	 * shows bad numbers for very large BSF code.
+	 */
+	u_int32_t 		l3proto_elems_ipv6[2];
+	struct {
+#define __FILTER_IPV6_MAX	20
+		u_int32_t 	addr[4];
+		u_int32_t 	mask[4];
+	} l3proto_ipv6[2][__FILTER_IPV6_MAX];
+
 	u_int32_t 		set[1];
 };
 
@@ -219,7 +267,8 @@ struct nf_expect {
 	struct nf_conntrack 	mask;
 	u_int32_t 		timeout;
 	u_int32_t 		id;
-	u_int16_t 		expectfn_queue_id;
+	u_int16_t		zone;
+	u_int32_t		flags;
 
 	u_int32_t 		set[1];
 };
