@@ -91,10 +91,10 @@ enum nf_conntrack_attr {
 	ATTR_DNAT_PORT,				/* u16 bits */
 	ATTR_TIMEOUT = 24,			/* u32 bits */
 	ATTR_MARK,				/* u32 bits */
-	ATTR_ORIG_COUNTER_PACKETS,		/* u32 bits */
-	ATTR_REPL_COUNTER_PACKETS,		/* u32 bits */
-	ATTR_ORIG_COUNTER_BYTES = 28,		/* u32 bits */
-	ATTR_REPL_COUNTER_BYTES,		/* u32 bits */
+	ATTR_ORIG_COUNTER_PACKETS,		/* u64 bits */
+	ATTR_REPL_COUNTER_PACKETS,		/* u64 bits */
+	ATTR_ORIG_COUNTER_BYTES = 28,		/* u64 bits */
+	ATTR_REPL_COUNTER_BYTES,		/* u64 bits */
 	ATTR_USE,				/* u32 bits */
 	ATTR_ID,				/* u32 bits */
 	ATTR_STATUS = 32,			/* u32 bits  */
@@ -124,6 +124,12 @@ enum nf_conntrack_attr {
 	ATTR_DCCP_STATE = 56,			/* u8 bits */
 	ATTR_DCCP_ROLE,				/* u8 bits */
 	ATTR_DCCP_HANDSHAKE_SEQ,		/* u64 bits */
+	ATTR_TCP_WSCALE_ORIG,			/* u8 bits */
+	ATTR_TCP_WSCALE_REPL = 60,		/* u8 bits */
+	ATTR_ZONE,				/* u16 bits */
+	ATTR_SECCTX,				/* string */
+	ATTR_TIMESTAMP_START,			/* u64 bits, linux >= 2.6.38 */
+	ATTR_TIMESTAMP_STOP = 64,		/* u64 bits, linux >= 2.6.38 */
 	ATTR_MAX
 };
 
@@ -193,10 +199,10 @@ extern void nfct_destroy(struct nf_conntrack *ct);
 struct nf_conntrack *nfct_clone(const struct nf_conntrack *ct);
 
 /* object size */
-extern size_t nfct_sizeof(const struct nf_conntrack *ct);
+extern __attribute__((deprecated)) size_t nfct_sizeof(const struct nf_conntrack *ct);
 
 /* maximum object size */
-extern size_t nfct_maxsize(void);
+extern __attribute__((deprecated)) size_t nfct_maxsize(void);
 
 /* set option */
 enum {
@@ -340,6 +346,9 @@ enum {
 
 	NFCT_OF_ID_BIT = 2,
 	NFCT_OF_ID = (1 << NFCT_OF_ID_BIT),
+
+	NFCT_OF_TIMESTAMP_BIT = 3,
+	NFCT_OF_TIMESTAMP = (1 << NFCT_OF_TIMESTAMP_BIT),
 };
 
 extern int nfct_snprintf(char *buf, 
@@ -399,6 +408,7 @@ enum {
 	NFCT_CP_ORIG = (1 << 0),
 	NFCT_CP_REPL = (1 << 1),
 	NFCT_CP_META = (1 << 2),
+	NFCT_CP_OVERRIDE = (1 << 3),
 };
 
 extern void nfct_copy(struct nf_conntrack *dest,
@@ -462,18 +472,21 @@ extern int nfct_filter_detach(int fd);
 
 /* low level API: netlink functions */
 
-extern int nfct_build_conntrack(struct nfnl_subsys_handle *ssh,
+extern __attribute__((deprecated)) int
+nfct_build_conntrack(struct nfnl_subsys_handle *ssh,
 				void *req,
 				size_t size,
 				u_int16_t type,
 				u_int16_t flags,
 				const struct nf_conntrack *ct);
 
-extern int nfct_parse_conntrack(enum nf_conntrack_msg_type msg,
+extern __attribute__((deprecated))
+int nfct_parse_conntrack(enum nf_conntrack_msg_type msg,
 				const struct nlmsghdr *nlh, 
 				struct nf_conntrack *ct);
 
-extern int nfct_build_query(struct nfnl_subsys_handle *ssh,
+extern __attribute__((deprecated))
+int nfct_build_query(struct nfnl_subsys_handle *ssh,
 			    const enum nf_conntrack_query query,
 			    const void *data,
 			    void *req,
@@ -492,6 +505,8 @@ enum nf_expect_attr {
 	ATTR_EXP_EXPECTED,	/* pointer to conntrack object */
 	ATTR_EXP_MASK,		/* pointer to conntrack object */
 	ATTR_EXP_TIMEOUT,	/* u32 bits */
+	ATTR_EXP_ZONE,		/* u16 bits */
+	ATTR_EXP_FLAGS,		/* u32 bits */
 	ATTR_EXP_MAX
 };
 
@@ -583,6 +598,27 @@ extern int nfexp_snprintf(char *buf,
 
 extern int nfexp_catch(struct nfct_handle *h);
 
+/* low level API */
+extern __attribute__((deprecated))
+int nfexp_build_expect(struct nfnl_subsys_handle *ssh,
+			      void *req,
+			      size_t size,
+			      u_int16_t type,
+			      u_int16_t flags,
+			      const struct nf_expect *exp);
+
+extern __attribute__((deprecated))
+int nfexp_parse_expect(enum nf_conntrack_msg_type type,
+			      const struct nlmsghdr *nlh,
+			      struct nf_expect *exp);
+
+extern __attribute__((deprecated))
+int nfexp_build_query(struct nfnl_subsys_handle *ssh,
+			     const enum nf_conntrack_query qt,
+			     const void *data,
+			     void *buffer,
+			     unsigned int size);
+
 /* Bitset representing status of connection. Taken from ip_conntrack.h
  * 
  * Note: For backward compatibility this shouldn't ever change
@@ -634,11 +670,24 @@ enum ip_conntrack_status {
 	IPS_DYING_BIT = 9,
 	IPS_DYING = (1 << IPS_DYING_BIT),
 
-    /* Connection has fixed timeout. */
+	/* Connection has fixed timeout. */
 	IPS_FIXED_TIMEOUT_BIT = 10,
 	IPS_FIXED_TIMEOUT = (1 << IPS_FIXED_TIMEOUT_BIT),
+
+	/* Conntrack is a template */
+	IPS_TEMPLATE_BIT = 11,
+	IPS_TEMPLATE = (1 << IPS_TEMPLATE_BIT),
+
+	/* Conntrack is a fake untracked entry */
+	IPS_UNTRACKED_BIT = 12,
+	IPS_UNTRACKED = (1 << IPS_UNTRACKED_BIT),
 };
 
+/* expectation flags */
+#define NF_CT_EXPECT_PERMANENT          0x1
+#define NF_CT_EXPECT_INACTIVE           0x2
+#define NF_CT_EXPECT_USERSPACE          0x4
+
 /*
  * TCP flags
  */
