@@ -1,5 +1,5 @@
 /*
- * (C) 2005 by Pablo Neira Ayuso <pablo@eurodev.net>
+ * (C) 2005 by Pablo Neira Ayuso <pablo@netfilter.org>
  *
  * This software may be used and distributed according to the terms
  * of the GNU General Public License, incorporated herein by reference.
@@ -8,6 +8,12 @@
 #ifndef _LIBNETFILTER_CONNTRACK_ICMP_H_
 #define _LIBNETFILTER_CONNTRACK_ICMP_H_
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/* WARNING: do not use these flags in your new applications, they are obsolete
+ * and we keep them here to avoid breaking backward compatibility. */
 enum icmp_flags {
 	ICMP_TYPE_BIT = 0,
 	ICMP_TYPE = (1 << ICMP_TYPE_BIT),
@@ -19,4 +25,8 @@ enum icmp_flags {
 	ICMP_ID = (1 << ICMP_ID_BIT)
 };
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
