@@ -167,6 +167,16 @@ static const void *get_attr_tcp_mask_repl(const struct nf_conntrack *ct)
 	return &ct->protoinfo.tcp.flags[__DIR_REPL].mask;
 }
 
+static const void *get_attr_tcp_wscale_orig(const struct nf_conntrack *ct)
+{
+	return &ct->protoinfo.tcp.wscale[__DIR_ORIG];
+}
+
+static const void *get_attr_tcp_wscale_repl(const struct nf_conntrack *ct)
+{
+	return &ct->protoinfo.tcp.wscale[__DIR_REPL];
+}
+
 static const void *get_attr_sctp_state(const struct nf_conntrack *ct)
 {
 	return &ct->protoinfo.sctp.state;
@@ -287,7 +297,42 @@ static const void *get_attr_helper_name(const struct nf_conntrack *ct)
 	return ct->helper_name;
 }
 
-get_attr get_attr_array[ATTR_MAX] = {
+static const void *get_attr_dccp_state(const struct nf_conntrack *ct)
+{
+	return &ct->protoinfo.dccp.state;
+}
+
+static const void *get_attr_dccp_role(const struct nf_conntrack *ct)
+{
+	return &ct->protoinfo.dccp.role;
+}
+
+static const void *get_attr_dccp_handshake_seq(const struct nf_conntrack *ct)
+{
+	return &ct->protoinfo.dccp.handshake_seq;
+}
+
+static const void *get_attr_zone(const struct nf_conntrack *ct)
+{
+	return &ct->zone;
+}
+
+static const void *get_attr_secctx(const struct nf_conntrack *ct)
+{
+	return ct->secctx;
+}
+
+static const void *get_attr_timestamp_start(const struct nf_conntrack *ct)
+{
+	return &ct->timestamp.start;
+}
+
+static const void *get_attr_timestamp_stop(const struct nf_conntrack *ct)
+{
+	return &ct->timestamp.stop;
+}
+
+const get_attr get_attr_array[ATTR_MAX] = {
 	[ATTR_ORIG_IPV4_SRC]		= get_attr_orig_ipv4_src,
 	[ATTR_ORIG_IPV4_DST] 		= get_attr_orig_ipv4_dst,
 	[ATTR_REPL_IPV4_SRC]		= get_attr_repl_ipv4_src,
@@ -344,4 +389,13 @@ get_attr get_attr_array[ATTR_MAX] = {
 	[ATTR_SCTP_VTAG_ORIG]		= get_attr_sctp_vtag_orig,
 	[ATTR_SCTP_VTAG_REPL]		= get_attr_sctp_vtag_repl,
 	[ATTR_HELPER_NAME]		= get_attr_helper_name,
+	[ATTR_DCCP_STATE]		= get_attr_dccp_state,
+	[ATTR_DCCP_ROLE]		= get_attr_dccp_role,
+	[ATTR_DCCP_HANDSHAKE_SEQ]	= get_attr_dccp_handshake_seq,
+	[ATTR_TCP_WSCALE_ORIG]		= get_attr_tcp_wscale_orig,
+	[ATTR_TCP_WSCALE_REPL]		= get_attr_tcp_wscale_repl,
+	[ATTR_ZONE]			= get_attr_zone,
+	[ATTR_SECCTX]			= get_attr_secctx,
+	[ATTR_TIMESTAMP_START]		= get_attr_timestamp_start,
+	[ATTR_TIMESTAMP_STOP]		= get_attr_timestamp_stop,
 };
