@@ -200,6 +200,20 @@ static void __parse_protoinfo_tcp(const struct nfattr *attr,
 		set_bit(ATTR_TCP_STATE, ct->set);
 	}
 
+	if (tb[CTA_PROTOINFO_TCP_WSCALE_ORIGINAL-1]) {
+		memcpy(&ct->protoinfo.tcp.wscale[__DIR_ORIG],
+		       NFA_DATA(tb[CTA_PROTOINFO_TCP_WSCALE_ORIGINAL-1]),
+		       sizeof(u_int8_t));
+		set_bit(ATTR_TCP_WSCALE_ORIG, ct->set);
+	}
+
+	if (tb[CTA_PROTOINFO_TCP_WSCALE_REPLY-1]) {
+		memcpy(&ct->protoinfo.tcp.wscale[__DIR_REPL],
+		       NFA_DATA(tb[CTA_PROTOINFO_TCP_WSCALE_REPLY-1]),
+		       sizeof(u_int8_t));
+		set_bit(ATTR_TCP_WSCALE_REPL, ct->set);
+	}
+
 	if (tb[CTA_PROTOINFO_TCP_FLAGS_ORIGINAL-1]) {
 		memcpy(&ct->protoinfo.tcp.flags[0], 
 		       NFA_DATA(tb[CTA_PROTOINFO_TCP_FLAGS_ORIGINAL-1]),
@@ -237,13 +251,40 @@ static void __parse_protoinfo_sctp(const struct nfattr *attr,
 	}
 
 	if (tb[CTA_PROTOINFO_SCTP_VTAG_REPLY-1]) {
-		ct->protoinfo.sctp.vtag[__DIR_ORIG] = 
+		ct->protoinfo.sctp.vtag[__DIR_REPL] = 
 			ntohl(*(u_int32_t *)NFA_DATA(tb[CTA_PROTOINFO_SCTP_VTAG_REPLY-1]));
-		set_bit(ATTR_SCTP_VTAG_ORIG, ct->set);
+		set_bit(ATTR_SCTP_VTAG_REPL, ct->set);
 	}
 
 }
 
+static void __parse_protoinfo_dccp(const struct nfattr *attr, 
+				   struct nf_conntrack *ct)
+{
+	struct nfattr *tb[CTA_PROTOINFO_DCCP_MAX];
+
+	nfnl_parse_nested(tb, CTA_PROTOINFO_DCCP_MAX, attr);
+
+	if (tb[CTA_PROTOINFO_DCCP_STATE-1]) {
+                ct->protoinfo.dccp.state =
+                        *(u_int8_t *)NFA_DATA(tb[CTA_PROTOINFO_DCCP_STATE-1]);
+		set_bit(ATTR_DCCP_STATE, ct->set);
+	}
+	if (tb[CTA_PROTOINFO_DCCP_ROLE-1]) {
+                ct->protoinfo.dccp.role =
+                        *(u_int8_t *)NFA_DATA(tb[CTA_PROTOINFO_DCCP_ROLE-1]);
+		set_bit(ATTR_DCCP_ROLE, ct->set);
+	}
+	if (tb[CTA_PROTOINFO_DCCP_HANDSHAKE_SEQ-1]) {
+		u_int64_t tmp;
+		memcpy(&tmp,
+		       NFA_DATA(tb[CTA_PROTOINFO_DCCP_HANDSHAKE_SEQ-1]),
+		       sizeof(tmp));
+		ct->protoinfo.dccp.handshake_seq = __be64_to_cpu(tmp);
+		set_bit(ATTR_DCCP_HANDSHAKE_SEQ, ct->set);
+	}
+}
+
 static void __parse_protoinfo(const struct nfattr *attr,
 			      struct nf_conntrack *ct)
 {
@@ -256,6 +297,9 @@ static void __parse_protoinfo(const struct nfattr *attr,
 
 	if (tb[CTA_PROTOINFO_SCTP-1])
 		__parse_protoinfo_sctp(tb[CTA_PROTOINFO_SCTP-1], ct);
+
+	if (tb[CTA_PROTOINFO_DCCP-1])
+		__parse_protoinfo_dccp(tb[CTA_PROTOINFO_DCCP-1], ct);
 }
 
 static void __parse_counters(const struct nfattr *attr,
@@ -272,10 +316,13 @@ static void __parse_counters(const struct nfattr *attr,
 				= ntohl(*(u_int32_t *)
 					NFA_DATA(tb[CTA_COUNTERS32_PACKETS-1]));
 
-		if (tb[CTA_COUNTERS_PACKETS-1])
-			ct->counters[dir].packets
-				= __be64_to_cpu(*(u_int64_t *)
-					NFA_DATA(tb[CTA_COUNTERS_PACKETS-1]));
+		if (tb[CTA_COUNTERS_PACKETS-1]) {
+			u_int64_t tmp;
+			memcpy(&tmp,
+			       NFA_DATA(tb[CTA_COUNTERS_PACKETS-1]),
+			       sizeof(tmp));
+			ct->counters[dir].packets = __be64_to_cpu(tmp);
+		}
 
 		switch(dir) {
 		case __DIR_ORIG:
@@ -293,10 +340,13 @@ static void __parse_counters(const struct nfattr *attr,
 				= ntohl(*(u_int32_t *)
 					NFA_DATA(tb[CTA_COUNTERS32_BYTES-1]));
 
-		if (tb[CTA_COUNTERS_BYTES-1])
-			ct->counters[dir].bytes
-				= __be64_to_cpu(*(u_int64_t *)
-					NFA_DATA(tb[CTA_COUNTERS_BYTES-1]));
+		if (tb[CTA_COUNTERS_BYTES-1]) {
+			u_int64_t tmp;
+			memcpy(&tmp,
+			       NFA_DATA(tb[CTA_COUNTERS_BYTES-1]),
+			       sizeof(tmp));
+			ct->counters[dir].bytes = __be64_to_cpu(tmp);
+		}
 
 		switch(dir) {
 		case __DIR_ORIG:
@@ -372,6 +422,20 @@ __parse_helper(const struct nfattr *attr, struct nf_conntrack *ct)
 	set_bit(ATTR_HELPER_NAME, ct->set);
 }
 
+static void
+__parse_secctx(const struct nfattr *attr, struct nf_conntrack *ct)
+{
+	struct nfattr *tb[CTA_SECCTX_MAX];
+
+	nfnl_parse_nested(tb, CTA_SECCTX_MAX, attr);
+	if (!tb[CTA_SECCTX_NAME-1])
+		return;
+
+	ct->secctx = strdup(NFA_DATA(tb[CTA_SECCTX_NAME-1]));
+	if (ct->secctx)
+		set_bit(ATTR_SECCTX, ct->set);
+}
+
 int __parse_message_type(const struct nlmsghdr *nlh)
 {
 	u_int16_t type = NFNL_MSG_TYPE(nlh->nlmsg_type);
@@ -389,8 +453,28 @@ int __parse_message_type(const struct nlmsghdr *nlh)
 	return ret;
 }
 
+static void
+__parse_timestamp(const struct nfattr *attr, struct nf_conntrack *ct)
+{
+	struct nfattr *tb[CTA_TIMESTAMP_MAX];
+
+	nfnl_parse_nested(tb, CTA_TIMESTAMP_MAX, attr);
+	if (tb[CTA_TIMESTAMP_START-1]) {
+		u_int64_t tmp;
+		memcpy(&tmp, NFA_DATA(tb[CTA_TIMESTAMP_START-1]), sizeof(tmp));
+		ct->timestamp.start = __be64_to_cpu(tmp);
+		set_bit(ATTR_TIMESTAMP_START, ct->set);
+	}
+	if (tb[CTA_TIMESTAMP_STOP-1]) {
+		u_int64_t tmp;
+		memcpy(&tmp, NFA_DATA(tb[CTA_TIMESTAMP_STOP-1]), sizeof(tmp));
+		ct->timestamp.stop = __be64_to_cpu(tmp);
+		set_bit(ATTR_TIMESTAMP_STOP, ct->set);
+	}
+}
+
 void __parse_conntrack(const struct nlmsghdr *nlh,
-		       const struct nfattr *cda[],
+		       struct nfattr *cda[],
 		       struct nf_conntrack *ct)
 {
 	struct nfgenmsg *nfhdr = NLMSG_DATA(nlh);
@@ -466,4 +550,15 @@ void __parse_conntrack(const struct nlmsghdr *nlh,
 
 	if (cda[CTA_HELP-1])
 		__parse_helper(cda[CTA_HELP-1], ct);
+
+	if (cda[CTA_ZONE-1]) {
+		ct->zone = ntohs(*(u_int16_t *)NFA_DATA(cda[CTA_ZONE-1]));
+		set_bit(ATTR_ZONE, ct->set);
+	}
+
+	if (cda[CTA_SECCTX-1])
+		__parse_secctx(cda[CTA_SECCTX-1], ct);
+
+	if (cda[CTA_TIMESTAMP-1])
+		__parse_timestamp(cda[CTA_TIMESTAMP-1], ct);
 }
