@@ -28,7 +28,7 @@ static const u_int8_t invmap_icmp[] = {
 #define ICMPV6_NI_REPLY 140
 #endif
 
-static u_int8_t invmap_icmpv6[] = {
+static const u_int8_t invmap_icmpv6[] = {
 	[ICMPV6_ECHO_REQUEST - 128]	= ICMPV6_ECHO_REPLY + 1,
 	[ICMPV6_ECHO_REPLY - 128]	= ICMPV6_ECHO_REQUEST + 1,
 	[ICMPV6_NI_QUERY - 128]		= ICMPV6_NI_QUERY + 1,
@@ -330,9 +330,24 @@ set_attr_dccp_handshake_seq(struct nf_conntrack *ct, const void *value)
 	ct->protoinfo.dccp.handshake_seq = *((u_int64_t *) value);
 }
 
+static void set_attr_tcp_wscale_orig(struct nf_conntrack *ct, const void *value)
+{
+	ct->protoinfo.tcp.wscale[__DIR_ORIG] = *((u_int8_t *) value);
+}
+
+static void set_attr_tcp_wscale_repl(struct nf_conntrack *ct, const void *value)
+{
+	ct->protoinfo.tcp.wscale[__DIR_REPL] = *((u_int8_t *) value);
+}
+
+static void set_attr_zone(struct nf_conntrack *ct, const void *value)
+{
+	ct->zone = *((u_int16_t *) value);
+}
+
 static void set_attr_do_nothing(struct nf_conntrack *ct, const void *value) {}
 
-set_attr set_attr_array[ATTR_MAX] = {
+const set_attr set_attr_array[ATTR_MAX] = {
 	[ATTR_ORIG_IPV4_SRC]	= set_attr_orig_ipv4_src,
 	[ATTR_ORIG_IPV4_DST] 	= set_attr_orig_ipv4_dst,
 	[ATTR_REPL_IPV4_SRC]	= set_attr_repl_ipv4_src,
@@ -380,11 +395,11 @@ set_attr set_attr_array[ATTR_MAX] = {
 	[ATTR_MASTER_L4PROTO]	= set_attr_master_l4proto,
 	[ATTR_SECMARK]		= set_attr_secmark,
 	[ATTR_ORIG_NAT_SEQ_CORRECTION_POS] 	= set_attr_orig_cor_pos,
-	[ATTR_ORIG_NAT_SEQ_OFFSET_BEFORE] 	= set_attr_orig_off_aft,
-	[ATTR_ORIG_NAT_SEQ_OFFSET_AFTER] 	= set_attr_orig_off_bfr,
+	[ATTR_ORIG_NAT_SEQ_OFFSET_BEFORE] 	= set_attr_orig_off_bfr,
+	[ATTR_ORIG_NAT_SEQ_OFFSET_AFTER] 	= set_attr_orig_off_aft,
 	[ATTR_REPL_NAT_SEQ_CORRECTION_POS] 	= set_attr_repl_cor_pos,
-	[ATTR_REPL_NAT_SEQ_OFFSET_BEFORE] 	= set_attr_repl_off_aft,
-	[ATTR_REPL_NAT_SEQ_OFFSET_AFTER] 	= set_attr_repl_off_bfr,
+	[ATTR_REPL_NAT_SEQ_OFFSET_BEFORE] 	= set_attr_repl_off_bfr,
+	[ATTR_REPL_NAT_SEQ_OFFSET_AFTER] 	= set_attr_repl_off_aft,
 	[ATTR_SCTP_STATE]	= set_attr_sctp_state,
 	[ATTR_SCTP_VTAG_ORIG]	= set_attr_sctp_vtag_orig,
 	[ATTR_SCTP_VTAG_REPL]	= set_attr_sctp_vtag_repl,
@@ -392,4 +407,10 @@ set_attr set_attr_array[ATTR_MAX] = {
 	[ATTR_DCCP_STATE]	= set_attr_dccp_state,
 	[ATTR_DCCP_ROLE]	= set_attr_dccp_role,
 	[ATTR_DCCP_HANDSHAKE_SEQ] = set_attr_dccp_handshake_seq,
+	[ATTR_TCP_WSCALE_ORIG]	= set_attr_tcp_wscale_orig,
+	[ATTR_TCP_WSCALE_REPL]	= set_attr_tcp_wscale_repl,
+	[ATTR_ZONE]		= set_attr_zone,
+	[ATTR_SECCTX]		= set_attr_do_nothing,
+	[ATTR_TIMESTAMP_START]	= set_attr_do_nothing,
+	[ATTR_TIMESTAMP_STOP]	= set_attr_do_nothing,
 };
