@@ -25,7 +25,7 @@ int __parse_expect_message_type(const struct nlmsghdr *nlh)
 }
 
 void __parse_expect(const struct nlmsghdr *nlh,
-		    const struct nfattr *cda[],
+		    struct nfattr *cda[],
 		    struct nf_expect *exp)
 {
 	struct nfgenmsg *nfhdr = NLMSG_DATA(nlh);
@@ -54,4 +54,15 @@ void __parse_expect(const struct nlmsghdr *nlh,
 		      ntohl(*(u_int32_t *)NFA_DATA(cda[CTA_EXPECT_TIMEOUT-1]));
 		set_bit(ATTR_EXP_TIMEOUT, exp->set);
 	}
+
+	if (cda[CTA_EXPECT_ZONE-1]) {
+		exp->zone =
+		      ntohs(*(u_int16_t *)NFA_DATA(cda[CTA_EXPECT_ZONE-1]));
+		set_bit(ATTR_EXP_ZONE, exp->set);
+	}
+	if (cda[CTA_EXPECT_FLAGS-1]) {
+		exp->flags =
+		      ntohl(*(u_int32_t *)NFA_DATA(cda[CTA_EXPECT_FLAGS-1]));
+		set_bit(ATTR_EXP_FLAGS, exp->set);
+	}
 }
