@@ -5,7 +5,7 @@
  * of the GNU General Public License, incorporated herein by reference.
  */
 
-#include "internal.h"
+#include "internal/internal.h"
 
 static void set_exp_attr_master(struct nf_expect *exp, const void *value)
 {
@@ -27,9 +27,21 @@ static void set_exp_attr_timeout(struct nf_expect *exp, const void *value)
 	exp->timeout = *((u_int32_t *) value);
 }
 
-set_exp_attr set_exp_attr_array[] = {
+static void set_exp_attr_zone(struct nf_expect *exp, const void *value)
+{
+	exp->zone = *((u_int16_t *) value);
+}
+
+static void set_exp_attr_flags(struct nf_expect *exp, const void *value)
+{
+	exp->flags = *((u_int32_t *) value);
+}
+
+const set_exp_attr set_exp_attr_array[ATTR_EXP_MAX] = {
 	[ATTR_EXP_MASTER]		= set_exp_attr_master,
 	[ATTR_EXP_EXPECTED]		= set_exp_attr_expected,
 	[ATTR_EXP_MASK]			= set_exp_attr_mask,
 	[ATTR_EXP_TIMEOUT]		= set_exp_attr_timeout,
+	[ATTR_EXP_ZONE]			= set_exp_attr_zone,
+	[ATTR_EXP_FLAGS]		= set_exp_attr_flags,
 };
