@@ -1,11 +1,13 @@
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <errno.h>
+#include <arpa/inet.h>
 
 #include <libnetfilter_conntrack/libnetfilter_conntrack.h>
 #include <libnetfilter_conntrack/libnetfilter_conntrack_tcp.h>
 
-int main()
+int main(void)
 {
 	int ret;
 	struct nfct_handle *h;
@@ -38,10 +40,13 @@ int main()
 
 	ret = nfct_query(h, NFCT_Q_UPDATE, ct);
 
-	printf("TEST: update conntrack (%d)(%s)\n", ret, strerror(errno));
-
+	printf("TEST: update conntrack ");
 	if (ret == -1)
-		exit(EXIT_FAILURE);
+		printf("(%d)(%s)\n", ret, strerror(errno));
+	else
+		printf("(OK)\n");
 
 	nfct_close(h);
+
+	ret == -1 ? exit(EXIT_FAILURE) : exit(EXIT_SUCCESS);
 }
