@@ -1,10 +1,12 @@
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <errno.h>
+#include <arpa/inet.h>
 
 #include <libnetfilter_conntrack/libnetfilter_conntrack.h>
 
-int main()
+int main(void)
 {
 	int ret;
 	struct nfct_handle *h;
@@ -18,11 +20,11 @@ int main()
 	}
 
 	nfct_set_attr_u8(expected, ATTR_L3PROTO, AF_INET);
-	nfct_set_attr_u32(expected, ATTR_IPV4_SRC, inet_addr("4.4.4.4"));
-	nfct_set_attr_u32(expected, ATTR_IPV4_DST, inet_addr("5.5.5.5"));
+	nfct_set_attr_u32(expected, ATTR_IPV4_SRC, inet_addr("1.1.1.1"));
+	nfct_set_attr_u32(expected, ATTR_IPV4_DST, inet_addr("2.2.2.2"));
 
 	nfct_set_attr_u8(expected, ATTR_L4PROTO, IPPROTO_TCP);
-	nfct_set_attr_u16(expected, ATTR_PORT_SRC, htons(10240));
+	nfct_set_attr_u16(expected, ATTR_PORT_SRC, 0);
 	nfct_set_attr_u16(expected, ATTR_PORT_DST, htons(10241));
 
 	exp = nfexp_new();
