@@ -1,5 +1,6 @@
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <errno.h>
 
 #include <libnetfilter_conntrack/libnetfilter_conntrack.h>
@@ -20,12 +21,14 @@ static int event_cb(enum nf_conntrack_msg_type type,
 	return NFCT_CB_CONTINUE;
 }
 
-int main()
+int main(void)
 {
 	int ret;
 	struct nfct_handle *h;
 
-	h = nfct_open(EXPECT, NF_NETLINK_CONNTRACK_EXP_NEW);
+	h = nfct_open(EXPECT, NF_NETLINK_CONNTRACK_EXP_NEW |
+			      NF_NETLINK_CONNTRACK_EXP_UPDATE |
+			      NF_NETLINK_CONNTRACK_EXP_DESTROY);
 	if (!h) {
 		perror("nfct_open");
 		return -1;
